#include <windows.h>
#include <stdio.h>

void PrintErrorMessage(char *szFile, char *szFunction, int nLine);
void PrintUsage();
BOOL SetPrivilege(HANDLE hToken, LPCTSTR lpszPrivilege, BOOL bEnablePrivilege);

int main(int argc, char *argv[]) {
	BOOL fSuspend;
	BOOL fForce;

	/* parse options */
	if ((argc != 2) && (argc != 3)) 
		PrintUsage();

	/* -H  -S tÕ`FbN */
	if (!strncmp(argv[1], "-H", 2))
		fSuspend = FALSE;
	else if (!strncmp(argv[1], "-S", 2))
		fSuspend = TRUE;
	else
		PrintUsage();

	/* -f tÕ`FbN */
	if (argc == 3)
		if (!strncmp(argv[2], "-f", 2))
			fForce = TRUE;

	/* SE_SHUTDOWN_NAME ̎擾 */
	{
		HANDLE hToken;
		/* vZXg[N̎擾 */
		if (!OpenProcessToken(GetCurrentProcess(), 
				TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) 
			PrintErrorMessage(__FILE__, "OpenProcessToken", __LINE__);
		/* SE_SHUTDOWN_NAME ̎擾 */
		if (!SetPrivilege(hToken, SE_SHUTDOWN_NAME, TRUE)) {
			CloseHandle(hToken);
			ExitProcess(1);
		}
		CloseHandle(hToken);
	}

	/* TXyh͋x~Ԃw */
	if (!SetSystemPowerState(fSuspend, fForce)) {
		PrintErrorMessage(__FILE__, "SetSystemPowerState", __LINE__);
		ExitProcess(1);
	}
	return 0;
}

void PrintUsage() {
	fprintf(stderr, "Usage: SetSystemState {-S|-H} [-f]\n");
	fprintf(stderr, "Call SetSystemPowerState(fSuspend, fForce)\n");
	fprintf(stderr, "   -S: Suspend, fSuspend = TRUE\n");
	fprintf(stderr, "   -H: Hibernate, fSuspend = FALSE\n");
	fprintf(stderr, "   -f: force, fForce = TRUE\n");
	ExitProcess(255);
}

/* Enabling and Disabling Privileges in C++
   http://msdn.microsoft.com/en-us/library/aa446619%28v=vs.85%29.aspx
*/

BOOL SetPrivilege(
    HANDLE hToken,          // access token handle
    LPCTSTR lpszPrivilege,  // name of privilege to enable/disable
    BOOL bEnablePrivilege   // to enable or disable privilege
    ) 
{
    TOKEN_PRIVILEGES tp;
    LUID luid;

    if ( !LookupPrivilegeValue( 
            NULL,            // lookup privilege on local system
            lpszPrivilege,   // privilege to lookup 
            &luid ) )        // receives LUID of privilege
    {
		PrintErrorMessage(__FILE__, "LookupPrivilegeValue", __LINE__);
//        printf("LookupPrivilegeValue error: %u\n", GetLastError() ); 
        return FALSE; 
    }

    tp.PrivilegeCount = 1;
    tp.Privileges[0].Luid = luid;
    if (bEnablePrivilege)
        tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    else
        tp.Privileges[0].Attributes = 0;

    // Enable the privilege or disable all privileges.

    if ( !AdjustTokenPrivileges(
           hToken, 
           FALSE, 
           &tp, 
           sizeof(TOKEN_PRIVILEGES), 
           (PTOKEN_PRIVILEGES) NULL, 
           (PDWORD) NULL) )
    { 
		PrintErrorMessage(__FILE__, "AdjustTokenPrivileges", __LINE__);
//          printf("AdjustTokenPrivileges error: %u\n", GetLastError() ); 
        return FALSE; 
    } 

    if (GetLastError() == ERROR_NOT_ALL_ASSIGNED)

    {
//          printf("The token does not have the specified privilege. \n");
		PrintErrorMessage(__FILE__, "AdjustTokenPrivileges", __LINE__);
          return FALSE;
    } 

    return TRUE;
}
